# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Util.pm -- Misc routines..
#

package Plugins::SrvrPowerCtrl::Util;

use base qw(Slim::Plugin::Base);
use strict;

use Slim::Utils::OSDetect;
use Time::Zone;
use File::Spec::Functions qw(:ALL);
use Cwd 'abs_path';
use Scalar::Util qw(blessed);

#use Net::Address::Ethernet qw( get_address );


#Global Variables..
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;

use Slim::Utils::Strings qw(string);


#sub max ($$) { $_[$_[0] < $_[1]] }
#
#sub min ($$) { $_[$_[0] > $_[1]] }

sub IsNull {
	if (!defined( @_) || ! @_ ) {
		return 1;
	}
	return 0;
}

sub IsNumeric {
	my ($num) = @_;

	#$g{log}->debug("Testing $num");

	if (!defined($num)) {
		return 0;
 	} elsif ($num =~ /^-?\d/) {
		return 1;
	}
	return 0;
}

sub IsValidClient {
	my $client = shift;
	if (eval{$client->id()}) {
		return 1;
	}
	return 0;
}

sub ClientAttribute {
	my ($client, $attr) = @_;
	return (eval {$client->$attr()} || "no $attr" );
	#return (eval {$client->{$attr}} || "no $attr" );
}

sub safename {
   my $clnt = shift;
   return (eval{$clnt->name()} || 'no name');
}

sub safeid {
   my $clnt = shift;
   return (eval{$clnt->id()} || 'no id');
}


sub GetPluginPath {
	my $fullname = File::Spec->rel2abs( __FILE__ );
	#my ($name,$path,$suffix) = fileparse($fullname,qr{\.pm});
	my ($volume, $path, $file) = File::Spec->splitpath($fullname);

	##Nope..this does not resolve symlinks..
	##$fullname = readlink($fullname);

	#Another aproach to resolve symlinks..
    my $absolute_path = $volume ? $volume . $path : $path;
    $path = abs_path($absolute_path);

	#test this with both 7.3.4 & 7.4
	#chop($path);

	#for windows..
	#use Win32::AbsPath qw(Relative2Absolute);
	#Relative2Absolute($path);

	#return $g{clsAppClass}->_pluginDataFor('basedir');
	return $path;
}


sub GetServerMacAddress {
	my $serverMAC;

	if ($^O =~ /^m?s?win/i) {		## Windows?
		$serverMAC = `ipconfig /all`;
	} else {						## Everything else..
		$serverMAC = `ifconfig -a`;
	}

	if ( $serverMAC =~ /((?:[0-9a-f]{2}[:-]){5}[0-9a-f]{2})/i ) {
		$serverMAC = $1;
		$serverMAC =~ s/\-/\:/g ;	## Windows ipconfig formats MAC as aa-bb-cc-dd-ee-ff
		return $serverMAC;
	}

	return 'UNKNOWN';
}


#Get the SqueezeCenter Version, change it from 7.3.4 form into 7.34 and make it numeric..
#LIMITATION: this would see 7.4.1 as equal to 7.4.10
sub GetSCVersion {
	my $nSCVersion = $::VERSION;
	if (length($nSCVersion) > 3) {
		my $szMinor = substr($nSCVersion, 3);
		#remove any periods..
		$szMinor =~ s/\.//g;
	   $nSCVersion = substr($nSCVersion, 0, 3) . $szMinor;
	   }

	$nSCVersion = $nSCVersion * 1;

	return $nSCVersion;
}

sub GetOSDistro {
	my $szDistro = Slim::Utils::OSDetect::getOS()->{osDetails}->{osName};
	
	#$g{log}->debug ("Raw distro name   == $szDistro");

	# clean up the name, removing version info..
	# e.g. on OS X, osName == MacOSX10.6.1(10B504)
	# Remove digits..
	$szDistro =~ s/\d//g;
	# Remove periods..
	$szDistro =~ s/\.//g;
	# Remove anything within parenthises
	$szDistro =~ s/\(.*\)//g;
	# Remove trailing whitespace and anything after that..
	#$szDistro =~ s/(^\S*)\s.*$/\1/g;
	$szDistro =~ s/\s//g;
	

	#$g{log}->debug ("Clean distro name == $szDistro");
	
	# from svn 28890 7.4 trunk, OSX.pm:
	#$szDistro =~ s/ \(\w+?\)$//;

	return $szDistro;
}


sub AnyPlayersPlaying {
	my @clients;
	my $curclient;

	@clients = Slim::Player::Client::clients();

	foreach $curclient (@clients) {
		#$g{log}->debug ($curclient->name() . " playmode is \[" . ClientAttribute($curclient, 'playmode') . "\]");
		if (Slim::Player::Source::playmode($curclient) eq 'play') {
			return 1;
		}
	}
	return 0;
}

sub AnyPlayersUpdating {
	my @clients;
	my $curclient;

	@clients = Slim::Player::Client::clients();

	foreach $curclient (@clients) {
		#Just block on isUpgrading, not on needsUpgrade..
		if ( $curclient->isUpgrading() ) {
			$g{log}->info ($curclient->name() . " needsUpgrade() is \[" . $curclient->needsUpgrade() . "\]");
			return 1;
		}
	}
	return 0;
}


sub DisplayPlayerMessage {
	my ($client, $message, $duration, $skipclient, $jivemessage) = @_;
	my @clients;
	my $curclient;
	my $numclients;

	if (!defined($duration)) {
		$duration = $g{prefs}->nRegretDelay;
	}

	if (IsValidClient($client)) {
		push(@clients, $client);
	} else {
		@clients = Slim::Player::Client::clients();
	}

	$numclients = @clients;

	#fixup the message..change underscores to spaces..
	$message =~ s/_/ /sig;

	#$g{log}->debug( "Displaying message \[" . $message . "\] on " . $numclients . " clients for " . $duration . " seconds.");

	foreach $curclient (@clients) {
		#Don't display a message here..
		#if ( defined($skipclient) && ($skipclient->id() eq $curclient->id()) ) {
		#if ( IsValidClient($skipclient) && ($skipclient->id() eq $curclient->id()) ) {
		if ( IsValidClient($skipclient) && ( ClientAttribute($skipclient, 'id') eq $curclient->id() ) ) {
			#$g{log}->debug( "Not displaying message on " . $curclient->name() );
			next;
		}

		#Don't bother trying to disply the message on a SBR or SBC..
		#if ($curclient->deviceid eq 7 || $curclient->deviceid eq 9) {
		if ($curclient->deviceid eq 7) {
			#$g{log}->debug("Skipping display on ". $curclient->name() );
			next;
		}

		#$g{log}->debug( "Displaying message on " . $curclient->name() );
		#Change the font on the BOOM:
		if ($curclient->deviceid == 10) {
			$curclient->showBriefly(
				{
				'center' => [ '', '0' ],
				'fonts' => {	'graphic-160x32' => { 'center' => [ 'standard_n.1', 'logoSB2.2' ] }, },
				'line1' => string('PLUGIN_SRVRPOWERCTRL_MODULE_NAME'),
				'line2' => $message
				},
				{
					'duration' => "$duration"
				} );

		} else {
			#$g{log}->debug("Displaying $message on ". $curclient->name() . "..");
			$curclient->showBriefly(
				{
					'line1' => string('PLUGIN_SRVRPOWERCTRL_MODULE_NAME'),
					'line2' => $message
				},
				{
					'duration' => "$duration"
				} );
		}

		#show on the SBC too..
		if (defined($jivemessage)) {
			$curclient->showBriefly( { 'jive' => { 'text'    => [ $jivemessage ], } },{'duration' => $duration, 'block' => 0, } );
		} else {
			$curclient->showBriefly( { 'jive' => { 'text'    => [ $message ], } },{'duration' => $duration, 'block' => 0, } );
		}

	}
}

sub BlockPlayerMessage {
	my ($client, $message) = @_;
	my @clients;
	my $curclient;
	my $line1;
	my $line2;

	#if (!IsNull($client)) {
	if (IsValidClient($client)) {
		push(@clients, $client);
	} else {
		@clients = Slim::Player::Client::clients();
	}

	#fixup the message..change underscores to spaces..
	$message =~ s/_/ /sig;

	$g{log}->debug( (IsValidClient($client) ? $client->name() : "All Clients") . ": " . ($message ? $message : "no message.") );

	foreach $curclient (@clients) {
		#$curclient->block( string('PLUGIN_SRVRPOWERCTRL_MODULE_NAME'), $message);

		#show on the SBC too..can't block here as it will block the controlled player too..
		$curclient->showBriefly( { 'jive' => { 'text'    => [ $message ], } },{'duration' => "15", 'block' => 0, } );

		$curclient->showBriefly(	{ 'line1' => string('PLUGIN_SRVRPOWERCTRL_MODULE_NAME'), 'line2' => $message },
									{	'duration' => "15",
										#'brightness' => "4",
										'block' => 1 },  );

	}

}

sub UnBlockPlayerMessage {
	my ($client, @args) = @_;
	my $message = $args[0];
	my $duration = $args[1];
	my @clients;
	my $curclient;

	$g{log}->debug( (IsValidClient($client) ? $client->name() : "All Clients") . ": " . ($message ? $message : "no message.") );

	if (!IsNull($client)) {
		push(@clients, $client);
	} else {
		@clients = Slim::Player::Client::clients();
	}

	foreach $curclient (@clients) {
		$curclient->unblock();
		$curclient->showBriefly( { 'jive' => { 'text'    => [ " " ], } },{'duration' => 10, 'block' => 0, } );
	}

	if (defined($message)) {
		$g{log}->debug($message);
		DisplayPlayerMessage($client, $message, $duration);
	}

}


sub PowerOffPlayer {
	my ($client) = @_;
	my @clients;
	my $curclient;

	if (IsValidClient($client)) {
		push(@clients, $client);
	} else {
		@clients = Slim::Player::Client::clients();
	}

	foreach $curclient (@clients) {
		$g{log}->debug("Switching player " . $curclient->name() . " to OFF");
		$curclient->execute(['power', '0']);
		#$curclient->power(0);
		#experiment with turning the brightness to 0
		#$g{log}->debug( "Trying to turn the " . $curclient->name() . " display dark..");
		#How do I check to see that a client poseses "brightness"?
		#$curclient->brightness(0);
		#$curclient->execute(['show', 'line1:Dark%20Display', 'line2:Should%20not%20see%20me', 'duration:30', 'centered:1', 'brightness:0']);

	}
}

sub RestartSC {
	$g{log}->debug("Attempting SC7.4 style service restart..");
	Slim::Control::Request::executeRequest(undef, ['restartserver']);
}

sub GetWhenShouldRun {
	my @aTime;
	my $szTime;

	@aTime = localtime(time());

	#forward 2 minutes..at on linux seems to need this..
	if ($aTime[0] >= 30) {
		$aTime[1] += 2;
	} else {
		#forward 1 minute..
		$aTime[1] += 1;
	}

	#we don't need no stinking seconds..
	$aTime[0] = 0;

	#fixup minutes..
	if ($aTime[1] >= 60) {
		if ($aTime[2] >= 23) {
			$aTime[2] = 0;
		}
		$aTime[1] -= 60;
	}

	$szTime = sprintf("%02d:%02d", $aTime[2], $aTime[1]);

	return $szTime;
}



# Quasi-printf processing of format placeholders in a command string..
# Substitution works as follows:
#
# %%	becomes		%
# %d	beccomes	unix epoch time (UTC)
# %l	becomes		unix epoch time + localtime offset
# %s	becomes		localtime 'YY-MM-DD HH:MM:SS'
# %f	becomes		localtime 'short-date', usu 'MM-DD-YY'
# %t	becomes		localtime 'HH:MM:SS'
# {+/-nnnn}			is used as a offset correction to the supplied time..

sub FormatCommand {
	my ($szCommand, $nTime, $nCorrection) = @_;

	if ( ! defined($szCommand) || ! length($szCommand) ) {
		return "";
	}

	if (!defined($nTime)) {
		$nTime = time();
	}

	if (!defined($nCorrection)) {
		$nCorrection = 0;
		#Anything within curly braces is interpreted as a request to correct the time, e.g. '{-120}' == subtract two minutes..
		if ( $szCommand =~ /\{(.+)\}/ ) {
			$nCorrection = $1;
			$szCommand =~ s/\{(.+)\}//g;		#remove the correction string..
			$g{log}->debug("Command $szCommand contained a correction value of $nCorrection..");
		}
	}

	$nTime += $nCorrection;

	my $nTimeLocal 	= $nTime + tz_local_offset($nTime);
	my $szDate	 	= Slim::Utils::DateTime::shortDateF($nTime);
	my $szTime	 	= Slim::Utils::DateTime::timeF($nTime, "%H:%M:%S");
	my $szDateTime 	= Slim::Utils::DateTime::timeF($nTime, "%Y-%m-%d %H:%M:%S");

	#make % replacements
	#$g{log}->debug("Before command: $szCommand");
	$szCommand =~ s/%%//g;

	$szCommand =~ s/%d/$nTime/g;		# time as unix epoch
	$szCommand =~ s/%l/$nTimeLocal/g;	# local unix epoch
	$szCommand =~ s/%s/$szDateTime/g;	# YY-MM-DD HH:MM:SS
	$szCommand =~ s/%f/$szDate/g;		# Localtime short-date format, usu MM-DD-YY
	$szCommand =~ s/%t/$szTime/g;		# Localtime HH:MM:SS

	$szCommand =~ s//%/g;

	#$g{log}->debug("After command:  $szCommand");

	return $szCommand;
}

# From Peter Watkin's KidsPlay plugin: parseFields()
# sub to handle quoted fields, e.g. 'playlist play "/path/with some spaces/playlist.m3u"'
# sub to handle quoted fields, e.g. 'playlist play "/path/with some spaces/playlist.m3u"'
sub parseFields($) {
        my $line = shift;
	# certain characters should be escaped with a \ :
	# 	\ ; " [ ] { }
	# if the \ char is followed by any other char,
	# \ and the char following are interpreted as 2 chars
	my $specialC = "\\;\"\[\]\{\}";
	$line =~ s/^\s*//;
	$line =~ s/\s*$//;
	my @cooked = ();
	my $in = 0;
	my $quoted = 0;
	my $escaped = 0;
	my $i = 0;
	my $word = '';
	#my $iswindows = $^O =~ /^m?s?win/i;

	while ($i < length($line) ) {
		my $c = substr($line,$i++,1);
		$escaped = 0;
		#if ( $c eq "\\" && !$iswindows ) {
		if ( $c eq "\\" ) {
			$escaped = 1;
			if ($i < (length($line) -1)) {
				my $c2 = substr($line,$i++,1);
				if (index($specialC,$c2) > -1) {
					$c = $c2;
				} else {
					--$i;
					$escaped = 0;
				}

			} else {
				# just a \
			}
		}
		if ( $in ) {
			if ( $escaped ) {
				$word .= $c;
			} else {
				# end of this word?
				if ( ($quoted && ($c eq '"')) || ((!$quoted) && ($c =~ /\s/)) ) {
					push @cooked, $word;
					$word = '';
					$in = 0;
					$quoted = 0;
				} else {
					# build & keep moving
					$word .= $c;
				}
			}
		} else {
			# look for delim
			if ( $c eq '"' ) {
				$quoted = 1;
				$in = 1;
			} elsif ( $c !~ /\s/ ) {
				$quoted = 0;
				$in = 1;
				$word .= $c;
			}
		}
	}
	if ( $in ) { push @cooked, $word; }
	return @cooked;
}

#also Peter Watkin's code..
sub splitLines($$) {
	my $macro = shift;
	my $delim = shift;
	my @ms;
	my $line = '';
	my $i = 0;
	while ($i < length($macro) ) {
		my $c = substr($macro,$i,1);
		if ( $c eq "\\" ) {
			if ($i < (length($macro) -1)) {
				$c .= substr($macro,++$i,1);
			} else {
				# invalid escape!
				$c = '';
			}
		} else {
			if ($c eq $delim) {
				push @ms, $line;
				$line = '';
				$c = '';
			}
		}
		$line .= $c;
		++$i;
	}
	if ($line ne '') {
		push @ms, $line;
	}
	return @ms;
}




sub XCLIExecCmd {
	my ($command, $time) = @_;
	my @clients;
	my $client = undef;
	my $playerMAC = undef;
	my $curclient;
	my $curclientname;
	my $macaddr;
	my @cmd;
	my $res;
	
	if (! $command =~ m/^cli\:\/\//i ) {
		return -1;
	}
	
	#strip off the 'cli://'
	$command = substr($command,6);

	#does the cli command begin with a mac address?
	
	if ( $command =~ /((?:[0-9a-f]{2}[:-]){5}[0-9a-f]{2})/i ) {
		$playerMAC = $1;
		$playerMAC =~ s/\-/\:/g ;	## Windows ipconfig formats MAC as aa-bb-cc-dd-ee-ff
		$command = substr($command, length($playerMAC));
		$command =~ s/^\s// ;

		@clients = Slim::Player::Client::clients();
		
		foreach $curclient (@clients) {
			$macaddr = $curclient->id();
			if ( defined($macaddr) && ( lc($macaddr) eq lc($playerMAC) ) ) {
				$client = $curclient;
				last;
			}
		}
	}

	$g{log}->info("Now trying to execute CLI " . ($playerMAC ? $playerMAC : 'undef') . " : [\'$command\']");
	#$res = Slim::Control::Request::executeRequest(undef, ["$command"]);
	@cmd = split(' ', $command );
	$res = Slim::Control::Request::executeRequest($client, [@cmd[0..$#cmd]]);
	#$res = Slim::Control::Request::executeRequest($playerMAC, [$command]);
	logHashElements($res);

	return $res;
#Function "Slim::Control::Request::executeRequest" accepts the usual parameters
#of client, command array and callback params, and returns the request object.

#my $request = Slim::Control::Request::executeRequest($client, ['stop']);

}

sub CLIExecCmd {
	my ($callingclient, $command, $time) = @_;
	my @commands;
	my $cmd;
	my $res;
	my @results;
	
	if (! $command =~ m/^cli\:\/\//i ) {
		return -1;
	}
	
	#strip off the 'cli://'
	$command = substr($command,6);

	#allow multi-command cli requests using && as a separator..
	@commands = split('&&', $command );
	
	foreach $cmd (@commands) {
		#strip leading whitespace..
		$cmd =~ s/^\s// ;

		# parse the command
		my ($client, $arrayRef) = Slim::Control::Stdio::string_to_array($cmd);

		if (!defined $arrayRef) {
			next;
		}

		$g{log}->info( ClientAttribute($callingclient, 'name') . ' requesting command: ' . ClientAttribute($client, 'id') . " @{$arrayRef}" );

		$res = Slim::Control::Request::executeRequest($client, $arrayRef);
		
		logHashElements( $res->{_results} );
		push(@results, $res);
	}
	return @results;	
}

sub SystemExecCmd {
	my ($client, $command, $time) = @_;
	my @commands;
	my $cmd;
	my @cmd_args;
	my $res;
	my @results = ();
	
	if (!$command) {
		return -1;
	}

	@commands = splitLines($command,";");
	
	$g{log}->debug( scalar(@commands) . " commands: $command");
	
	foreach $cmd (@commands) {
		
		$cmd =~ s/^\s*//;
		$cmd =~ s/\s*$//;
		
		$cmd = FormatCommand( $cmd, $time );

		if ( $cmd =~ m/^\s*cli\:\/\//i ) {
			$res = CLIExecCmd($client, $cmd, $time);
		} else {
			@cmd_args = parseFields($cmd);
			$g{log}->info( ClientAttribute($client, 'name') . ' executing command: ' . "@cmd_args" );
			$res = system @cmd_args[0..$#cmd_args];
			#$res = "We are not doing this!";
			#sudo return values:
			#
			#Upon successful execution of a program, the exit status from sudo will simply be
			#the exit status of the program that was executed.
			#
			#Otherwise, sudo quits with an exit value of 1 if there is a configuration/permission
			#problem or if sudo cannot execute the given command. In the latter case the error string is printed to stderr.
			#$res == 1 may indicate a sudo permissions problem...not sure about return values from cmd.exe..
			$g{log}->debug("Command returned $res");
			push(@results, $res);
		}
	}


	return @results;
}



sub ScheduleCommand {
	my ($client, $item) = @_;

	#Find the next alarm and program the RTC to wake up the server at the appropriate time.
	if ($item->{'setrtcwakeup'}) {
		Plugins::SrvrPowerCtrl::Alarms::SetRTCWakeup();
	}

	#Do we want to call Slim::Utils::Timers::pendingTimers() here to see if there is anything to wait for?

	#cue up our action command to execute in the future..
	return Slim::Utils::Timers::setTimer(undef, time() + $item->{'cmdwait'},
		sub {
			my $res;
			my $command;
			my $cmdargs;
			my @cmd;

			#We're the timer action...clear the timer ID since it's fired..and the saved pending action..
			$g{tPendingActionTimer} = 0;
			$g{hPreviousAction} = $g{hPendingAction};
			$g{hPendingAction}->{isSleepDefered} = 0;
			$g{hPendingAction} = { };

			$command = $item->{command};

			# execute the coderef (if any) to get args for the cmd..
			if ( defined($item->{getcommandargcoderef}) ) {
				#execute the coderef..
				$cmdargs = &{$item->{getcommandargcoderef}}($client, $item);
				if ( defined($cmdargs) ) {
					if ( (IsNumeric($cmdargs)) && ($cmdargs == -1) ) {
						#fail mode..
						my $failmsg = sprintf( string('PLUGIN_SRVRPOWERCTRL_FAILURE_MSG'),  ucfirst( lc($item->{action}) ) );
						$g{log}->error("$item->{action} $item->{getcommandargcoderef} failed!");
						UnBlockPlayerMessage(undef, $failmsg);
						return;
					} else {
						#anything else returned is a cmdline arg for our script..
						$command = $item->{command} . ' ' . $cmdargs;
					}
				} else {
					#the coderef returned undef..
					$command = $item->{command};
				}
			}

			#if the command is a coderef rather than a script or system command name, execute it..
			if (ref($item->{command}) eq 'CODE') {
				$g{log}->debug("Command is a coderef!");
				#execute the coderef..
				$cmdargs = &{$item->{command}}($client, $item);
			#else the command is a system command or script..
			} else {
				SystemExecCmd($client, $command);
			}

			if ($item->{'stopsc'}) {
				if (!IsNull($client)) {
					#from SqueezeTray.pl
					$client->execute(['stopserver']);
				} else {
					main::stopServer();
				}
			}

			if (!$item->{dispblock}) {
				UnBlockPlayerMessage();
			}

		}, );
}

sub PushToSqueezeNetwork {
	my ($client) = @_;
	my @clients;
	my $curclient;
	my $curclientname;
	my $macaddr;
	my $specificmacaddr;
	my $szSNPushedPlayers;
	my @aSNPushedPlayers = ();
	my $szNextAlarmClientMAC;
	my $szMACPushList;
	my @aMACPushList;
	my $nMACPushList = 0;

	#get the mac address of the next alarm owner..
	my ($nRTCWakeupTime, $szNextAlarmClientMAC) = Plugins::SrvrPowerCtrl::Alarms::GetNextAlarm();

	my $snserver = Slim::Networking::SqueezeNetwork->get_server('sn');
	$g{log}->debug( "SBS reports get_server(\'sn\') == $snserver");
	
	@clients = ();

	#Get our list of specific MACs to restore from SN from the prefs..
	$szMACPushList = $g{prefs}->szPushSNPlayerMACs;
	#delimiters could be ; , or space
	@aMACPushList = split(/[\;\,\s]/, $szMACPushList );
	$nMACPushList = scalar @aMACPushList || 0;

	#Did we get an array of MACs to push from the prefs?
	if($nMACPushList) {
		$g{log}->debug( "szMACPushList: $szMACPushList == $nMACPushList players..");
	}

	#Are we pushing just a single client??
	if ( (!$g{prefs}->bGotoSNAllPlayers) && $client ) {
		push( @clients, $client );
	} elsif ($nMACPushList) {
		@clients = Slim::Player::Client::clients();
	} else {
		@clients = Slim::Player::Client::clients();
	}

	#Are we pushing only specific players?
	if ( $nMACPushList > 0 ) {
		$g{log}->debug( "Trying to switch $nMACPushList players to SN..");
		foreach $specificmacaddr (@aMACPushList) {
			if (defined($specificmacaddr)) {
				foreach $curclient (@clients) {
					$macaddr = $curclient->id();
					if (defined($macaddr)) {
						if (lc($macaddr) eq lc($specificmacaddr)) {
							$curclientname = $curclient->name() || "unknown";
							$g{log}->debug( "Attempting to connect " . $curclient->name() . ": " . $curclient->id() . " to $snserver..");
							$curclient->execute(['connect', $snserver]);
							push( @aSNPushedPlayers, $curclient->id() );
						}
					}
				}
			}
		}
	} else {
		#push the players over to SN
		foreach $curclient (@clients) {
			$curclientname = $curclient->name();

			#For now, don't push Controllers and the like over to SqueezeNetwork..
			#!!!!!!!!!!!!!!!!!FIX THIS!!!!!!!!!!!!!!!
			#Should put some version checking in here...7.4 probably will support
			#SqueezePlay audio playback of SqueezeNetwork...
			if ( $g{nSCVersion} >= 7.4 && $curclient->isa('Slim::Player::SqueezePlay') ) {
			#if ( $curclient->isa('Slim::Player::SqueezePlay') ) {
			#if ( $curclient->deviceid eq 9	) {
				$g{log}->debug( "Not attempting to connect " . $curclientname . ": " . $curclient->id() . " to $snserver..");
			} else {
				$g{log}->debug( "Attempting to connect " . $curclientname . ": " . $curclient->id() . " to $snserver..");
				$curclient->execute(['connect', $snserver]);
				push( @aSNPushedPlayers, $curclient->id() );
			}
		}
	}

	#Check the mac of the next alarm..  If it's not in our list, then the player was already on SN.
	foreach my $szCurMAC (@aSNPushedPlayers) {
		if ( lc($szCurMAC) eq lc($szNextAlarmClientMAC) ) {
			$szNextAlarmClientMAC = undef;
		}
	}

	#Add that mac to our list of players to be pulled back on wakeup..
	if (defined($szNextAlarmClientMAC)) {
		$g{log}->debug( "Adding $szNextAlarmClientMAC to our list of pushed players because it has an alarm scheduled..");
		push( @aSNPushedPlayers, $szNextAlarmClientMAC );
	}

	#Save the list of pushed players in the prefs for later restore..
	$szSNPushedPlayers = join(';', @aSNPushedPlayers);
	$g{prefs}->set('szSNPushedPlayers',	$szSNPushedPlayers );

	return undef;
}

#Get an array of the players currently connected to SqueezeNetwork MACs..
sub GetSNPlayers {
	my @aSNPlayerMACs;
	#Any way to get this to work with test.mysqueezenetwork.com ?
	my @aSNPlayers = Slim::Networking::SqueezeNetwork::Players->get_players();
	my $nSNPlayerCount = scalar @aSNPlayers  || 0;

	my $szDebugMsg = sprintf("nSNPlayerCount==%d", $nSNPlayerCount);
	$g{log}->debug ("$szDebugMsg");

	foreach my $player (@aSNPlayers ) {
		$g{log}->debug ("Player connected to SN: " . $player->{mac} . " - " . $player->{name} );
		push(@aSNPlayerMACs, $player->{mac});
	}

	return @aSNPlayerMACs;
}

sub PullFromSqueezeNetwork {
	my $szMACRestoreList;
	my @aMACRestoreList;
	my $nMACRestoreList = 0;
	my $request;
	my @aSNPlayerMACs;
	my $nSNPlayerMACs = 0;
	my $nDisconnectCount = 0;
	my $specificmacaddr;
	my $macaddr;
	my $nSNPushedPlayers;
	my $szSNPushedPlayers;
	my @aSNPushedPlayers = ();
	my $snserver = Slim::Networking::SqueezeNetwork->get_server('sn');
	
	$g{log}->debug( "SBS reports get_server(\'sn\') == $snserver");
	

	#if ( !$g{prefs}->bUseOnWakeupWatchdog ) {
	#	$g{log}->debug( "Need to activate OnWakeupWatchdog..");
	#	$g{prefs}->set('bUseOnWakeupWatchdog',	1 );
	#	Plugins::SrvrPowerCtrl::Watchdog::ActivateOnWakeupWatchdog(1);
	#}

	#Get our list of specific MACs to restore from SN from the prefs..
	$szMACRestoreList = $g{prefs}->szPullSNPlayerMACs;
	#delimiters could be ; , or space
	@aMACRestoreList = split(/[\;\,\s]/, $szMACRestoreList );
	$nMACRestoreList = scalar @aMACRestoreList || 0;

	#Did we get an array of MACs to restore from the prefs?
	if($nMACRestoreList) {
		$g{log}->debug( "szMACRestoreList: $szMACRestoreList == $nMACRestoreList players..");
	}

	#Get our list of MACs that we previously pushed onto SN..
	$szSNPushedPlayers = $g{prefs}->szSNPushedPlayers;
	#delimiters could be ; , or space
	@aSNPushedPlayers = split(/[\;\,\s]/, $szSNPushedPlayers );
	$nSNPushedPlayers = scalar @aSNPushedPlayers || 0;
	if($nSNPushedPlayers) {
		$g{log}->debug( "szSNPushedPlayers: $szSNPushedPlayers == $nSNPushedPlayers players..");
	}

	#Get the players currently connected to SN..
	@aSNPlayerMACs = GetSNPlayers();
	$nSNPlayerMACs = scalar @aSNPlayerMACs || 0;

	#if we don't have any players connected to SN, give up..
	if ($nSNPlayerMACs == 0) {
		$g{log}->debug( "No players to restore..");
		#Erase the list of pushed players..
		$g{prefs}->set('szSNPushedPlayers',	'');
		return;
	}

	#Are we pulling only specific players?
	if ( $nMACRestoreList > 0 ) {
		$g{log}->debug( "Trying to restore only $nMACRestoreList players from SN..");
		foreach $specificmacaddr (@aMACRestoreList) {
			if (defined($specificmacaddr)) {
				$g{log}->debug( "Checking player $specificmacaddr to see if it's connected..");
				foreach $macaddr (@aSNPlayerMACs) {
					if (defined($macaddr)) {
						if (lc($macaddr) eq lc($specificmacaddr)) {
							$g{log}->debug( "Attempting to disconnect $macaddr from $snserver...");
							$request = Slim::Control::Request::executeRequest(undef, [ 'squeezenetwork', 'disconnect', $macaddr ]);
						}
					}
				}
			}
		}
	} else {
		$g{log}->debug( "Trying to restore ALL pushed players from SN..");

		if (!defined(@aSNPushedPlayers) ) {
			$g{log}->debug( "No pushed player list!");
			@aSNPushedPlayers = @aSNPlayerMACs;
		}
		#Restore only the players we pushed onto SN when we suspended..
		foreach $specificmacaddr (@aSNPushedPlayers) {
			if (defined($specificmacaddr)) {
				$g{log}->debug( "Checking player $specificmacaddr to see if it's connected..");
				foreach $macaddr (@aSNPlayerMACs) {
					if (defined($macaddr)) {
						if (lc($macaddr) eq lc($specificmacaddr)) {
							$g{log}->debug( "Attempting to disconnect $macaddr from $snserver...");
							$request = Slim::Control::Request::executeRequest(undef, [ 'squeezenetwork', 'disconnect', $macaddr ]);
						}
					}
				}
			}
		}
	}

	#Erase the list of pushed players..
	$g{prefs}->set('szSNPushedPlayers',	'');

}


sub TestCoderefFail {
	my ($client, $item) = @_;

	$g{log}->debug("Testing failure mode..");

	if (!IsNull($client)) {
		$g{log}->debug("client == " . $client->name() . ", type == " . $client->deviceid );
	} else {
		$g{log}->debug("client == noclient");
	}

	$g{log}->debug("action == " . $item->{action} );

	return -1;
}

sub logHashElements {

	my @myhash = shift;
	my $num = 0;

	foreach my $item (@myhash) {
		$g{log}->debug("item\[$num\]:");
		while( my ($k, $v) = each %$item ) {
 			$g{log}->debug("key: $k, value: " . ( defined($v) ? $v : "undef" ) );
    		}
		$num++;
	}
}

sub logArgs {
	my (@args) = @_;
	my $n = 0;

	while (defined($args[$n])) {
		$g{log}->debug("arg[$n] == $args[$n++]");
	}
}

1;
