# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Watchdog.pm -- watchdog routines..support for shutting down server if players have been idle
#                   for a user-settable time period..
#


package Plugins::SrvrPowerCtrl::Watchdog;

use base qw(Slim::Plugin::Base);
use Slim::Utils::Strings qw(string);
use strict;

use Slim::Utils::DateTime;
use Slim::Utils::Validate;

#Global Variables..
use Plugins::SrvrPowerCtrl::Settings ();
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;

#Globals for this module..
my $nTimerInterval = 60;		#number of seconds between watchdog checks..should be 60.
my $nWatchdogActionDelay = 5;	#number of seconds to wait before immediate action..

my $bUseOnWakeupWatchdog = 0;	#Not indicated in prefs anymore...this gets enabled by other watchdogs..
my $bUseIdleWatchdog = 0;		#Used so that IdleWatchdog can be temporarily disabled..
my $bSkipEODCheck = 0;			#Don't check EOD if we start durring EOD.

my $nEODPlayersTimeCount = 0;	#EOD idle player countdown..
my $nIdlePlayersTimeCount = 0;	#Idle player countdown..
my $nLastOnWakeupTimeCheck = 0;	#Last wakeup time-check..



#sub min ($$) { $_[$_[0] > $_[1]] }

#sub max ($$) { $_[$_[0] < $_[1]] }


sub ActivateWatchdogs {

	# Activate the SleepPlayDone watchdog..
	if ($g{prefs}->bUseSleepEndWatchdog) {
		ActivateSleepEndWatchdog($g{prefs}->bUseSleepEndWatchdog);
	}

	# Activate the end-of-day idle-player watchdog..
	if ($g{prefs}->bUseEODWatchdog) {
		ActivateEODWatchdog($g{prefs}->bUseEODWatchdog);
	}

	# Activate the idle-player watchdog..
	if ($g{prefs}->bUseIdleWatchdog) {
		ActivateIdleWatchdog($g{prefs}->bUseIdleWatchdog);
	}

	# Activate the OnWakeup watchdog..
	if ($bUseOnWakeupWatchdog) {
		ActivateOnWakeupWatchdog($bUseOnWakeupWatchdog);
	}

}


sub CueWatchdogAction {
	my $item = shift;

	#use this as an alternative to Plugins::SrvrPowerCtrl::Plugin::prepareAction() since
	# we don't need a cancel time-out in this context and we don't want to advertise too
	# heavily on the device screens..

	#Don't prepare an action if one is already pending..
	if ($g{tPendingActionTimer}) {
		$g{log}->debug("Timer # $g{tPendingActionTimer} already pending.  Action " . $item->{action} . " ignored..");
		return 0;
	}

	#Don't restart the countdown here.  The OnWakeupWatchdog will reset this upon eventual return from suspend/hibernate/SN..
	#$nIdlePlayersTimeCount = 0;

	#Temporarily disable the idle-watchdog...let the wakeup-watchdog or the PlayRequestsMonitor re-enable it..
	$bUseIdleWatchdog = 0;

	#cue up the requested action..
	$g{tPendingActionTimer} = Slim::Utils::Timers::setTimer( undef, time() + $nWatchdogActionDelay, \&Plugins::SrvrPowerCtrl::Plugin::performAction, ( $item ) );
	if ($g{tPendingActionTimer}) {
		$g{log}->debug("Timer # $g{tPendingActionTimer} created.  Action " . $item->{action} . " is pending in $nWatchdogActionDelay seconds..");
		$item->{exeTime} = time() + $nWatchdogActionDelay;
		$g{hPendingAction} = $item;
		Plugins::SrvrPowerCtrl::Util::logHashElements($g{hPendingAction});
		return 1;
	}
	return 0;
}

# This is a way to re-activate the IdleWatchdog if our OnIdle or OnEODIdle actions don't actually
# put the system to sleep.

sub PlayRequestMonitor {
	my $request = shift;
	my $client;
	my $cmd;
	my @filter = ('play', 'pause', 'playlist newsong');

	$cmd = $request->getRequestString();
	$client = $request->client() || "unknown";

	# randomplay; playlist clear; *playlist newsong*; playlist loadtracks

	if ( grep { $_ eq $cmd} @filter ) {

		$g{log}->debug("MonitorPlayRequests called for client: ". $client->name() || 'no client' . ",   Filtered RequestString: $cmd");
		#Plugins::SrvrPowerCtrl::Util::logHashElements($request);

		#reset the OnIdle counts..
		$nIdlePlayersTimeCount = 0;
		$nEODPlayersTimeCount = 0;

		#clear the previous action..
		$g{hPreviousAction} = ( );

		#re-enable the IdleWatchdog..
		if ( $g{prefs}->bUseIdleWatchdog  && ! $bUseIdleWatchdog ) {
			$bUseIdleWatchdog = 1;
		}

    } #else {
		#$g{log}->debug("MonitorPlayRequests called for client: ". $client->name() || 'no client' . ", Unfiltered RequestString: $cmd");
	#}

	return 0;
}

my $bPlayMonitorActive = 0;		#check to see that we don't activate / deactivate multiple times...

# Called from WakeupWatchdog if the IdleWatchdog is disabled and if system hasn't been to sleep.

sub ActivatePlayRequestMonitor {
	my $bEnable = shift;

	if ($bEnable && ! $bPlayMonitorActive) {
		Slim::Control::Request::subscribe( \&PlayRequestMonitor, [['play', 'pause']]);
		$bPlayMonitorActive = 1;
		$g{log}->debug("PlayRequestMonitor activated!");
	} elsif (!$bEnable && $bPlayMonitorActive) {
		Slim::Control::Request::unsubscribe( \&PlayRequestMonitor, [['play', 'pause']]);
		$bPlayMonitorActive = 0;
		$g{log}->debug("PlayRequestMonitor deactivated!");
	}
	return $bEnable && $bPlayMonitorActive ;
}


sub ActivateIdleWatchdog {
	my $bEnable = shift;

	#reset the count..
	$nIdlePlayersTimeCount = 0;

	if ($bEnable){
		$bUseOnWakeupWatchdog = 1;
		$bUseIdleWatchdog = 1;
	} else {
		$bUseIdleWatchdog = 0;
	}

	return $bEnable;
}

sub GetIdleCount {
	return ($nIdlePlayersTimeCount, $nEODPlayersTimeCount);
}

sub SetIdleCount {
	($nIdlePlayersTimeCount, $nEODPlayersTimeCount) = @_;
	return ($nIdlePlayersTimeCount, $nEODPlayersTimeCount);
}

sub ResetIdleCount {
	$nIdlePlayersTimeCount = 0;
	$nEODPlayersTimeCount = 0;
	return ($nIdlePlayersTimeCount, $nEODPlayersTimeCount);
}

sub IdleWatchdog {
	my $nCurTime;

	my $item;
	my @clients;
	my $curclient;
	my $action;
	my $nTimeDelay;

	if (!$bUseIdleWatchdog) {
		$g{log}->debug("HEY! IdleWatchdog is supposed to be disabled!");
	}

	#De-activate the PlayRequestMonitor..
	if ($bPlayMonitorActive) {
		ActivatePlayRequestMonitor(0);
	}

	#$g{log}->debug("Checking to see if scanner is running & if players are idle..");
	if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersPlaying() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
		# We're playing, or scanning, or etc..
		$g{log}->debug("Players are not idle..");
		#Not idle...reset the timer to check again..
		$nIdlePlayersTimeCount = 0;

		if ($g{prefs}->szNotIdleWatchdog_cmd) {
			Plugins::SrvrPowerCtrl::Util::SystemExecCmd(undef, $g{prefs}->szNotIdleWatchdog_cmd);
		}

		return 0;
	}

	#We're idle..increment the idle count..
	$nIdlePlayersTimeCount++;

	#Has the idle time-out period been exceeded?
	if ( $nIdlePlayersTimeCount < $g{prefs}->nIdleWatchdogTimeout ) {
		# We're idle, counting down..
		$g{log}->debug("Players are idle: " . ($g{prefs}->nIdleWatchdogTimeout - $nIdlePlayersTimeCount) . " minutes left in the idle countdown..");

		if ($g{prefs}->szNotIdleWatchdog_cmd) {
			Plugins::SrvrPowerCtrl::Util::SystemExecCmd(undef, $g{prefs}->szNotIdleWatchdog_cmd);
		}

	} else {
		# We're idle and the countdown has expired
		$g{log}->debug("Players are idle and idle timeout of $nIdlePlayersTimeCount has elapsed..");

		$action = $g{prefs}->szIdleWatchdogAction;

		#This is just for testing...not a useful watchdog action..
		if ($action eq 'plugin_menu') {
			@clients = Slim::Player::Client::clients();
			foreach $curclient (@clients) {
				Plugins::SrvrPowerCtrl::Plugin::oursleepButtonHandler( $curclient );
			}
		return;
		}

		$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
		if (!defined($item)) {
			$g{log}->error( "Bad IdleWatchdog Action: $action");
			return;
		}

		#cue up the requested action..
		return CueWatchdogAction( $item );

	}
	return 1;
}


sub TimeStrToSeconds {
	my $szTime = shift;
	my $seconds;

	if (  ( !defined($szTime) ) ||
	      ( !defined(Slim::Utils::Validate::isTime($szTime)) )
		) {
		return -1;
	}

	$seconds = Slim::Utils::DateTime::prettyTimeToSecs($szTime);

	if (!$seconds) {
		return -1;
	}

	return $seconds;
}

sub IsInEOD {
	my $nCurTime = shift || time();
	my $nEODStartSeconds;
	my $nEODEndSeconds;
	my $nNowSeconds;

	if (!$g{prefs}->bUseEODWatchdog) {
		return 0;
	}

	#Get the seconds since midnight..
	my ($lt_sec,$lt_min,$lt_hour) = localtime($nCurTime);
	$nNowSeconds = $lt_sec + ($lt_min * 60) + ($lt_hour * 3600);

	#calculate the nEODStartSeconds..
	$nEODStartSeconds =  TimeStrToSeconds($g{prefs}->szEODWatchdogStartTime);
	if ($nEODStartSeconds == -1) {
		return -1;
	}

	#calculate the nEODEndSeconds..
	$nEODEndSeconds =  TimeStrToSeconds($g{prefs}->szEODWatchdogEndTime);
	if ($nEODEndSeconds == -1) {
		return -1;
	}

	#if we start before midnight and end tomorrow...
	if ( ($nEODStartSeconds > $nEODEndSeconds) && (($nNowSeconds >= $nEODStartSeconds) || ($nNowSeconds <= $nEODEndSeconds)) ) {
		return 1;
	} elsif ( ($nNowSeconds >= $nEODStartSeconds) && ($nNowSeconds <= $nEODEndSeconds) ) {
		return 1;
	}

	return 0;
}

sub TimeToStartOfEOD {
	my $nCurTime = shift || time();
	my ($lt_sec,$lt_min,$lt_hour) = localtime($nCurTime);
	my $nNowSeconds;
	my $nEODStartSeconds;

	#get the seconds since last midnight..
	$nNowSeconds = $lt_sec + ($lt_min * 60) + ($lt_hour * 3600);

	#calculate the nEODStartSeconds..
	$nEODStartSeconds =  TimeStrToSeconds($g{prefs}->szEODWatchdogStartTime);
	if ($nEODStartSeconds == -1) {
		return -1;
	}

	#if we are past start of EOD today..
	if ($nNowSeconds > $nEODStartSeconds) {
		return 86400 - $nNowSeconds + $nEODStartSeconds;
	} else {
		return $nEODStartSeconds - $nNowSeconds;
	}

	return 0;
}


sub ActivateEODWatchdog {
	my $bEnable = shift;

	$bSkipEODCheck = ( IsInEOD() && (!$g{nEODWatchdogTimeout}) ) ? 1 : 0;

	if ($bSkipEODCheck) {
		$g{log}->debug("Skipping EOD checks because IsInEOD and bSkipEODCheck == " . $bSkipEODCheck );
	} elsif ($bEnable){
		$nEODPlayersTimeCount = 0;
		$bUseOnWakeupWatchdog = 1;
	}

	return $bEnable;
}



#Another tweak to the EOD behavior that I'm thinking of trying is this:
#
#If the EOD idle timeout is set to zero, then the custom script gets executed just
#once within that EOD period. All other EOD idle invocations would ignore the custom
#script and fire the selected 'stock' action instead. This would allow the custom script
#to fire once and perform the big chores. Then, with SC restarted, the next EOD action
#could be a quick suspend. Thus the server would be awake for a shorter time for the chores.


sub EODWatchdog {
	my $nCurTime = shift;

	my $item;
	my @clients;
	my $curclient;
	my $action;
	my $nTimeDelay;

	#if we have an idle period to check..
	if ($g{prefs}->nEODWatchdogTimeout) {
		if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersPlaying() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
			#Not idle...reset the timer to check again..
			$g{log}->debug("Players are not idle..");
			$nEODPlayersTimeCount = 0;
			if ($g{prefs}->bUseIdleWatchdog && $g{prefs}->szNotIdleWatchdog_cmd) {
				Plugins::SrvrPowerCtrl::Util::SystemExecCmd(undef, $g{prefs}->szNotIdleWatchdog_cmd);
			}
			return 0;
		}
	} else {
		#No idle period..skip players playing check..
		if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
			#Not idle...reset the timer to check again..
			$g{log}->debug("Players are not idle..");
			$nEODPlayersTimeCount = 0;
			if ($g{prefs}->bUseIdleWatchdog && $g{prefs}->szNotIdleWatchdog_cmd) {
				Plugins::SrvrPowerCtrl::Util::SystemExecCmd(undef, $g{prefs}->szNotIdleWatchdog_cmd);
			}
			return 0;
		}
	}

	#OK, we're idle..increment the idle count..
	$nEODPlayersTimeCount++;

	#Are we still in the timeout grace period?
	if ( $nEODPlayersTimeCount < $g{prefs}->nEODWatchdogTimeout ) {
		#Still counting down...
		$g{log}->debug("Players are idle: " . ($g{prefs}->nEODWatchdogTimeout - $nEODPlayersTimeCount) . " minutes left in the EOD idle countdown..");
		if ($g{prefs}->bUseIdleWatchdog && $g{prefs}->szNotIdleWatchdog_cmd) {
			Plugins::SrvrPowerCtrl::Util::SystemExecCmd(undef, $g{prefs}->szNotIdleWatchdog_cmd);
		}

	} else {
		#No, time's up..

		$g{log}->debug("Players are idle and EOD idle timeout of $nEODPlayersTimeCount has elapsed..");

		$action = $g{prefs}->szEODWatchdogAction;

		#This is just for testing...not a useful watchdog action..
		if ($action eq 'plugin_menu') {
			@clients = Slim::Player::Client::clients();
			foreach $curclient (@clients) {
				Plugins::SrvrPowerCtrl::Plugin::oursleepButtonHandler( $curclient );
			}
		return;
		}

		#Alternate EOD action command takes precedence..
		if ( defined($g{prefs}->szEODWatchdog_cmd) && length($g{prefs}->szEODWatchdog_cmd) > 0 ) {
			$action = 'specialcmd';
			$item =  ( {	action					=> $action,
							actionid				=> 0xFFFFFFFF,
							menuindex				=> -1,
							menutext				=> string('PLUGIN_SRVRPOWERCTRL_SPECIALCMD'),
							message					=> string('PLUGIN_SRVRPOWERCTRL_SPECIALCMD_MSG'),
							dispblock				=> 1,
							getcommandargcoderef	=> undef,
							poweroffplayers			=> $g{prefs}->bPowerOffPlayers,
							gotosn					=> 0,
							checkblock				=> 1,
							cancelwait				=> 0,
							cmdwait					=> 5,
							refreshwait				=> -1,
							stopsc					=> 0,
							setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
							isSleepDefered			=> 0,
							command					=> $g{prefs}->szEODWatchdog_cmd } ) ;
		} else {
			$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
		}

		if (!defined($item)) {
			$g{log}->error( "BadEODWatchdog Action: $action");
			return;
		}
		#cue up the requested action..
		return CueWatchdogAction( $item );
	}
	return 1;
}


sub ActivateOnWakeupWatchdog {
	my $bEnable = shift;

	if ($bEnable) {
		$nLastOnWakeupTimeCheck = time();
		$bUseOnWakeupWatchdog	= 1;
	} else {
		$nLastOnWakeupTimeCheck = 0;
		#We have to really, really, really mean it in order to disable this..
		if (! $g{prefs}->bUseSleepEndWatchdog &&
			! $g{prefs}->bUseEODWatchdog &&
			! $g{prefs}->bUseIdleWatchdog &&
			! $g{prefs}->szOnWakeup_cmd ) {
			$bUseOnWakeupWatchdog	= 0;
		}
	}

	return $bEnable;
}


sub OnWakeupWatchdog {
	my $nCurTime = shift;
	my $szCommand;

	#have we returned from a suspend or hibernation or has system time changed??
	if ( ($nCurTime > ($nLastOnWakeupTimeCheck + $nTimerInterval + 30)) || $nCurTime < $nLastOnWakeupTimeCheck ) {
		$g{log}->info("Wakeup Call!!");

		#reset the OnIdle counts..
		$nIdlePlayersTimeCount = 0;
		$nEODPlayersTimeCount = 0;

		#clear the previous action..
		$g{hPreviousAction} = ( );

		#perform our OnWakup cmd..
		$szCommand = $g{prefs}->szOnWakeup_cmd;

		Plugins::SrvrPowerCtrl::Util::SystemExecCmd(undef, $szCommand, $nCurTime);

		# Pull back any players from SqueezeNetwork that we want..
		if ($g{prefs}->bRetrievePlayersFromSN) {
			Slim::Utils::Timers::setTimer(undef, time() + $g{prefs}->nRetrievePlayersFromSNWait, \&Plugins::SrvrPowerCtrl::Util::PullFromSqueezeNetwork, );
		}

		#re-active the idle-watchdog..
		if ($g{prefs}->bUseIdleWatchdog) {
			ActivateIdleWatchdog($g{prefs}->bUseIdleWatchdog);
		}

	} else {
		#$g{log}->debug("But I haven't been asleep!");
		#Activate the PlayRequestMonitor if the IdleWatchdog is temporarily disabled..
		if ( $g{prefs}->bUseIdleWatchdog && !$bUseIdleWatchdog ) {
			ActivatePlayRequestMonitor(1);
		}

	}

	$nLastOnWakeupTimeCheck = $nCurTime;

}

my $bSleepTimerIsActive = 0;

sub SleepRequestMonitor {
	my $request = shift;
	my $client;
	my $cmd;
	my $nSleepTime;
	my $nCurrentSleepTime;

	$cmd = $request->getRequestString();
	
	$g{log}->debug("cmd == $cmd");

	if ($cmd ne 'sleep') {
		return 0;
	}

	$client = $request->client() || return 0;

	$nSleepTime = $client->sleepTime();
	$nCurrentSleepTime = $client->currentSleepTime();

	$g{log}->debug("MonitorSleepRequests called for client: ". $client->name() || 'no client' . ", cmd: $cmd, sleepTime: $nSleepTime, currentSleepTime: $nCurrentSleepTime");

	#if $nSleepTime || $nCurrentSleepTime are 0, the the user has canceled sleep..and we should cancel end of sleep action..
	if (!$nSleepTime || !$nCurrentSleepTime){
		$g{log}->debug("Sleep timer canceled!");
		$bSleepTimerIsActive = 0;
		#Cancel any pending action..
		if (defined($g{tPendingActionTimer})) {
			Plugins::SrvrPowerCtrl::Plugin::cancelAction(undef, $g{hPendingAction});
		}
		ActivateSleepRequestMonitor(0);
	} else {
		$g{log}->debug("Sleep timer detected!");
		$bSleepTimerIsActive = 1;
		if ( $g{tPendingActionTimer} ) {
			Plugins::SrvrPowerCtrl::Plugin::resecheduleAction(undef, $g{hPendingAction});
		}

	}

}

my $bSleepRequestMonitorIsActive = 0;

sub ActivateSleepRequestMonitor {
	my $bEnable = shift;

	if ($bEnable && !$bSleepRequestMonitorIsActive){
		Slim::Control::Request::subscribe( \&SleepRequestMonitor, [['sleep']]);
		$bSleepRequestMonitorIsActive = 1;
		$g{log}->debug("SleepRequestMonitor activated!");
	} elsif (!$bEnable && $bSleepRequestMonitorIsActive) {
		Slim::Control::Request::unsubscribe( \&SleepRequestMonitor, [['sleep']]);
		$bSleepRequestMonitorIsActive = 0;
		$g{log}->debug("SleepRequestMonitor deactivated!");
	}

	return $bEnable;
}

sub ActivateSleepEndWatchdog {
	my $bEnable = shift;

	$bSleepTimerIsActive = 0;

	$bUseOnWakeupWatchdog = $bEnable;

	#This was commented out.  Why?
	ActivateSleepRequestMonitor($bEnable);

	return $bEnable;
}

sub SleepEndWatchdog {
	my $nCurTime = shift;
	my $action;
	my $item;
	my $nSleepTime = Plugins::SrvrPowerCtrl::SleepButton::GetSleepTime();

	if ($nSleepTime && !$bSleepTimerIsActive) {
		$bSleepTimerIsActive = 1;
		ActivateSleepRequestMonitor(1);
		return 0;
	}

	if (!$nSleepTime && $bSleepTimerIsActive) {
		#we've transitioned to end of sleep..
		$g{log}->debug("End of sleep timer detected!");

		#restart the sleep timer active flag for our eventual return from suspend/hibernate/SN..
		$bSleepTimerIsActive = 0;

		#check to see that all other players are idle before taking action...
		if ( Plugins::SrvrPowerCtrl::Block::IsBlocked() || Plugins::SrvrPowerCtrl::Util::AnyPlayersPlaying() || Plugins::SrvrPowerCtrl::Util::AnyPlayersUpdating() || Slim::Music::Import->stillScanning() ) {
			$g{log}->debug("Not idle condition..end-of-sleep action not taken!");
			return 0;
		}

		$action = $g{prefs}->szSleepEndWatchdogAction;

		$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
		if (!defined($item)) {
			$g{log}->error( "Bad SleepEndWatchdog Action: $action");
			return 0;
		}

		#Temporarily disable the idle-watchdog...let the wakeup-watchdog or the PlayRequestsMonitor re-enable it..
		$bUseIdleWatchdog = 0;

		#Stop monitoring sleep requests...
		ActivateSleepRequestMonitor(0);

		#cue up the requested action..
		$g{log}->error( "SleepEndWatchdog Action: $action");
		return CueWatchdogAction( $item );
	}
	return 1;
}


sub ActivateSPCWatchdog {
	my $bEnable = shift;
	my $szTimerName = 'srvrpowerctrl.watchdog';
	my $szParamValue = 'multi.check';

	if ($bEnable) {
		$g{tSPCWatchdogTimer} = Slim::Utils::Timers::setTimer($szTimerName, time() + $nTimerInterval, \&SPCWatchdog, ($szParamValue), );
	} else {
		if (defined($g{tSPCWatchdogTimer})) {
			Slim::Utils::Timers::killSpecific($g{tSPCWatchdogTimer});
			$g{tSPCWatchdogTimer} = 0;
		}

		if (defined($g{tPendingActionTimer})) {
			Slim::Utils::Timers::killSpecific($g{tPendingActionTimer});
			$g{tPendingActionTimer} = 0;
		}
	}
}



#This is the main timer routine that checks the various watchdog conditions..
sub SPCWatchdog {
	my ($szTimerName, $param1, $param2, $param3) = @_;
	my $nCurTime = time();
	my $bIsInEOD = IsInEOD($nCurTime);
	my $szCommand;
	my $nIdleFailSafe;

	#$g{log}->debug("$szTimerName, $param1");

	#Only check watchdogs if we aren't waiting for a cancel action..
	#This might be a problem...no NotIdle action is taken during a pending action during sleep-playing..
	if ( !$g{tPendingActionTimer} || $g{hPendingAction}->{isSleepDefered} ) {
		#Check our OnWakekup conditions..
		if ( $bUseOnWakeupWatchdog ) {
			OnWakeupWatchdog($nCurTime);
		}

		#Check the SleepDone watchdog..
		if ( $g{prefs}->bUseSleepEndWatchdog ) {
			SleepEndWatchdog($nCurTime);
		}

		#Check the End-of-day watchdog..
		if ( $bIsInEOD ) {
			#if we aren't skipping this check..
			if ( !$bSkipEODCheck ) {
				EODWatchdog($nCurTime);
			}
		} else {
			#reset the skip flag for next EOD
			$bSkipEODCheck = 0;
			#Check the OnIdle watchdog..but not if it's been temporarily disabled..
			if ( $g{prefs}->bUseIdleWatchdog && $bUseIdleWatchdog ) {
				IdleWatchdog($nCurTime);
			}
		}
	}

	#Schedule next check..
	$g{tWatchdogTimer} = Slim::Utils::Timers::setTimer($szTimerName, $nCurTime + $nTimerInterval, \&SPCWatchdog, ($param1), );
}

1;
